
# q-expansion of newform 1728.4.a.w, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1728
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [4], [-3], [-28], [11], [-44], [29], [172], [192], [-116], [69], [-384], [328], [156], [-392], [-412], [425], [257], [-1000], [-359], [-877], [328], [1572], [-1483], [-576], [-79], [1836], [-1450], [-636], [1460], [-464], [-36], [-1149], [1736], [-737], [-2854], [-2013], [1212], [1920], [-3240], [-4545], [2148], [-841], [1524], [5321], [-3497], [-1804], [1320], [-4114], [-5008], [-2536], [-4287], [-1424], [5848], [-3800], [1820], [1615], [-754], [544], [848], [-6708], [2112], [-5820], [-7413], [-8856], [3131], [-6819], [11304], [-2235], [-296], [5700], [-6943], [877], [10361], [5184], [8260], [5098], [-3192], [-639], [-11340], [115], [-3796], [-6062], [10548], [-15416], [-12820], [470], [12796], [-14603], [12084], [-1528], [8117], [-16596], [-13440], [8652], [4884], [18028], [20593], [7443], [-23653], [-12356], [8288], [-13228], [-2811], [8963], [22844], [17528], [15168], [22906], [-17219], [20569], [17772], [-11099], [25895], [9080], [-22632], [-30112], [25368], [-13080], [9433], [14275], [-28268], [5840], [-12376], [-11076], [-7253], [28456], [10428], [23206], [-3424], [11692], [-29349], [13555], [-10316], [-145], [33576], [-26795], [-4784], [-18488], [33304], [43380], [-33785], [-38732], [8255], [36360], [40525], [45192], [-4425], [32380], [32565], [-40052], [26891], [-46168], [47449], [-8528], [45596], [-2744], [-10389], [-55188], [22044], [-4140], [18139], [23340], [39096], [1092], [-5887], [35426], [42269], [-29588], [42600], [7674], [25052], [-33491], [-42023], [22600], [47312], [-23940], [54083], [-9830], [-13205], [-49928], [3198], [7836], [-14640], [-24600], [-30121], [-4185], [44821], [-28068], [-24071], [-18648], [-37073], [13180], [15036], [-38084], [-52973], [-6313], [-13484], [-31752], [41180], [-44033], [-77345], [59437], [-39024], [-22440], [66775], [-80460], [-4820], [-10343], [-12585], [48516], [25691], [21556], [8336], [-49897], [-32564], [-35740], [71496], [31848], [-23897], [90036], [-68304], [-93308], [-36468], [-30527], [17124], [-59940], [-47247], [-71836], [33315], [-62648], [-24716], [70268], [43071], [-80140], [-57211], [24028], [-23096], [-28040], [19608], [85133], [-21217], [-76171], [38772], [-95740], [-42877], [83904], [99616], [-4336], [-18042], [97788], [-7444], [3013], [-60732], [-47652], [73879], [-18080], [46848], [-76358], [-29947], [55692], [-76813], [64857], [-23560], [-12976], [87116], [-98744], [43592], [-102916], [116633], [71111], [-23738], [38483], [-15458], [-96047], [-93228], [-150470], [-85334], [8240], [-62276], [-17116], [6880], [6325], [-11809], [44596], [-27247], [79560], [105349], [-118004], [44068], [-79920], [-125320], [66048], [-41190], [-152096], [112573], [76836], [-39312], [-26035], [69899], [136064], [-115748], [65696], [55023], [-15922], [876], [13315], [33440], [15565], [-163044], [-107568], [-50480], [140419], [120724], [130790], [19380], [-102660], [155582], [3468], [51985], [110899], [-141736], [96092], [-61192], [34485], [-68760], [-10056], [9412], [-16452], [4715], [80208], [173367], [101328], [-121392], [-86232], [-17491], [63948], [151094], [-164844], [210005], [137436], [-43416], [-33813], [-86192], [200684], [124110], [168976], [189240], [106916], [136509], [13313], [111232], [-48396], [24613], [201532], [134824], [99088], [-196720], [100296], [152809], [-132336], [46776], [182932], [-102161], [-166377], [206268], [-222787], [90319], [-175028], [-89385], [137312], [-88304], [-99811], [41371], [-31260], [-45216], [83527], [140856], [-232819], [199524], [-110720], [-91137], [206376], [115789], [-63528], [-133044], [-204329], [-82373], [-3324], [69886], [14272], [-154236], [119325], [69612], [-19511], [212871], [10428], [-15256], [-90992], [160954], [-101712], [-51004], [-255584], [-17624], [-201364], [150863], [-17880], [187307], [139068], [24050], [-73688], [-9644], [127507], [169545], [-78452], [13884], [-137177], [-235200], [31160], [98612], [42274], [-119284], [286384], [6970], [-293440], [42164], [103656], [-283176], [-164048]]
